// Copyright 2021 gRPC authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef GRPC_SRC_CORE_LIB_DEBUG_HISTOGRAM_VIEW_H
#define GRPC_SRC_CORE_LIB_DEBUG_HISTOGRAM_VIEW_H

#include <stdint.h>

#include <grpc/support/port_platform.h>

namespace grpc_core {

struct HistogramView {
  int (*bucket_for)(int value);
  const int* bucket_boundaries;
  int num_buckets;
  const uint64_t* buckets;

  double Percentile(double p) const;
  double Count() const;
  double ThresholdForCountBelow(double count_below) const;
};

}  // namespace grpc_core

#endif  // GRPC_SRC_CORE_LIB_DEBUG_HISTOGRAM_VIEW_H
