/* This file was generated by upb_generator from the input file:
 *
 *     envoy/config/accesslog/v3/accesslog.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#ifndef ENVOY_CONFIG_ACCESSLOG_V3_ACCESSLOG_PROTO_UPB_MINITABLE_H_
#define ENVOY_CONFIG_ACCESSLOG_V3_ACCESSLOG_PROTO_UPB_MINITABLE_H_

#include "upb/generated_code_support.h"

// Must be last.
#include "upb/port/def.inc"

#ifdef __cplusplus
extern "C" {
#endif

extern const upb_MiniTable envoy__config__accesslog__v3__AccessLog_msg_init;
extern const upb_MiniTable envoy__config__accesslog__v3__AccessLogFilter_msg_init;
extern const upb_MiniTable envoy__config__accesslog__v3__ComparisonFilter_msg_init;
extern const upb_MiniTable envoy__config__accesslog__v3__StatusCodeFilter_msg_init;
extern const upb_MiniTable envoy__config__accesslog__v3__DurationFilter_msg_init;
extern const upb_MiniTable envoy__config__accesslog__v3__NotHealthCheckFilter_msg_init;
extern const upb_MiniTable envoy__config__accesslog__v3__TraceableFilter_msg_init;
extern const upb_MiniTable envoy__config__accesslog__v3__RuntimeFilter_msg_init;
extern const upb_MiniTable envoy__config__accesslog__v3__AndFilter_msg_init;
extern const upb_MiniTable envoy__config__accesslog__v3__OrFilter_msg_init;
extern const upb_MiniTable envoy__config__accesslog__v3__HeaderFilter_msg_init;
extern const upb_MiniTable envoy__config__accesslog__v3__ResponseFlagFilter_msg_init;
extern const upb_MiniTable envoy__config__accesslog__v3__GrpcStatusFilter_msg_init;
extern const upb_MiniTable envoy__config__accesslog__v3__MetadataFilter_msg_init;
extern const upb_MiniTable envoy__config__accesslog__v3__LogTypeFilter_msg_init;
extern const upb_MiniTable envoy__config__accesslog__v3__ExtensionFilter_msg_init;

extern const upb_MiniTableFile envoy_config_accesslog_v3_accesslog_proto_upb_file_layout;

#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port/undef.inc"

#endif  /* ENVOY_CONFIG_ACCESSLOG_V3_ACCESSLOG_PROTO_UPB_MINITABLE_H_ */
